package ch.psi.daq.rest.config;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.util.StringUtils;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hazelcast.client.HazelcastClient;
import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.core.ManagedContext;
import com.hazelcast.spring.context.SpringManagedContext;

import ch.psi.daq.common.statistic.StorelessStatistics;
import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.hazelcast.config.HazelcastConfig;
import ch.psi.daq.rest.ResponseStreamWriter;
import ch.psi.daq.rest.model.PropertyFilterMixin;

@Configuration
@PropertySource(value = { "classpath:rest.properties" })
@PropertySource(value = { "file:${user.home}/.config/daq/rest.properties" }, ignoreResourceNotFound = true)
public class RestConfig {

   private static final Logger logger = LoggerFactory.getLogger(RestConfig.class);
   
   // TODO refactor - also defined in HazelcastNodeConfig
   private static final String HAZELCAST_GROUP_PASSWORD = "hazelcast.group.password";
   private static final String HAZELCAST_GROUP_NAME = "hazelcast.group.name";
   
   @Autowired
   private Environment env;
   
   // a nested configuration
   // this guarantees that the ordering of the properties file is as expected
   // see:
   // https://jira.spring.io/browse/SPR-10409?focusedCommentId=101393&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-101393
   // @Configuration
   // @Import(CassandraConfig.class)
   // static class InnerConfiguration { }
   
   @Bean
   public JsonFactory jsonFactory() {
      return new JsonFactory();
   }
   
   @Bean
   public ResponseStreamWriter responseStreamWriter() {
      return new ResponseStreamWriter();
   }
   
   @Bean(name = HazelcastConfig.BEAN_NAME_HAZELCAST_CLIENT)
   public HazelcastInstance hazelcastClientInstance() {
      return HazelcastClient.newHazelcastClient(hazelcastClientConfig());
   }

   private ClientConfig hazelcastClientConfig() {
      ClientConfig config = new ClientConfig();
      config.setManagedContext(managedContext());

      config.getNetworkConfig().setAddresses(hazelcastInitialCandidates());
      
      String groupName = env.getProperty(HAZELCAST_GROUP_NAME);
      if (groupName != null) {
         config.getGroupConfig().setName(groupName);
      }
      String groupPassword = env.getProperty(HAZELCAST_GROUP_PASSWORD);
      if (groupPassword != null) {
         config.getGroupConfig().setPassword(groupPassword);
      }

      return config;
   }
   
   
   private ManagedContext managedContext() {
      return new SpringManagedContext();
   }

   private List<String> hazelcastInitialCandidates() {
      List<String> clusterMembers = Arrays.asList(StringUtils.commaDelimitedListToStringArray(env.getProperty("hazelcast.initialcandidates")));
      logger.info("The following hosts have been defined to form a Hazelcast cluster: {}", clusterMembers);
      return clusterMembers;
   }

   @Bean
   public ObjectMapper objectMapper() {
      ObjectMapper mapper = new ObjectMapper();
      // only include non-null values
      mapper.setSerializationInclusion(Include.NON_NULL);
      // Mixin which is used dynamically to filter out which properties get serialised and which
      // won't. This way, the user can specify which columns are to be received.
      mapper.addMixIn(ChannelEvent.class, PropertyFilterMixin.class);
      mapper.addMixIn(StorelessStatistics.class, PropertyFilterMixin.class);
      return mapper;
   }

}
